<?php

namespace Drupal\cache_debugger\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Cache Debugger.
 *
 * @package Drupal\cache_debugger\Form
 */
class CacheDebugger extends ConfigFormBase {

  const SERVICES_YML = DRUPAL_ROOT . '/sites/default/services.yml';
  const DEFAULT_SERVICES_YML = DRUPAL_ROOT . '/sites/default/default.services.yml';
  const SITES_DEFAULT_DIR = DRUPAL_ROOT . '/sites/default';
  const PERMISSION_WRITE = 0777;
  const PERMISSION_READ = 0555;

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return ['cache_debugger.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cache_debugger_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('cache_debugger.settings');

    $form['description'] = [
      '#plain_text' => $this->t('This setting should only be enabled on development environments and never on production.'),
    ];

    $form['cache_debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable render cache debugging'),
      '#default_value' => $config->get('cache_debug'),
      '#description' => $this->t('This setting will add cache debugging output for each rendered element.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('cache_debugger.settings')
      ->set('cache_debug', $form_state->getValue('cache_debug'))
      ->save();

    if ($form_state->getValue('cache_debug')) {
      $this->enableCacheDebug();
    }
    else {
      $this->disableCacheDebug();
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * Enables cache debugging by updating the services.yml file.
   */
  private function enableCacheDebug() {
    if (!file_exists(self::SERVICES_YML)) {
      $this->prepareServicesYml();
      $this->updateServicesYml('debug: false', 'debug: true');
      drupal_flush_all_caches();
    }
  }

  /**
   * Disables cache debugging by removing the services.yml file.
   */
  private function disableCacheDebug() {
    if (file_exists(self::SERVICES_YML)) {
      $this->removeServicesYml();
      drupal_flush_all_caches();
    }
  }

  /**
   * Copies the default services.yml file and sets permissions.
   */
  private function prepareServicesYml() {
    chmod(self::SITES_DEFAULT_DIR, self::PERMISSION_WRITE);
    touch(self::SERVICES_YML);
    copy(self::DEFAULT_SERVICES_YML, self::SERVICES_YML);
    chmod(self::SITES_DEFAULT_DIR, self::PERMISSION_READ);
  }

  /**
   * Updates the services.yml file by replacing a specific string.
   *
   * @param string $search
   *   The string to search for in the file.
   * @param string $replace
   *   The string to replace the search string with.
   */
  private function updateServicesYml($search, $replace) {
    $file_contents = file_get_contents(self::SERVICES_YML);
    $search_position = strrpos($file_contents, $search);
    $updated_contents = substr($file_contents, 0, $search_position) . $replace . substr($file_contents, $search_position + strlen($search));
    file_put_contents(self::SERVICES_YML, $updated_contents);
  }

  /**
   * Removes the services.yml file.
   */
  private function removeServicesYml() {
    chmod(self::SITES_DEFAULT_DIR, self::PERMISSION_WRITE);
    unlink(self::SERVICES_YML);
    chmod(self::SITES_DEFAULT_DIR, self::PERMISSION_READ);
  }

}
